using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib13;

namespace XNA_SceneNode
{
    public class Model
    {
        private UWB_SceneNode mScene;
        private UWB_SceneNode mLeftPalmNode, mRightPalmNode;
        private UWB_SceneNode mActiveNode;

        private UWB_PrimitiveList mBullets;
        private float mSecondCounter;
        private int mHitCount;

        private UWB_XNADrawHelper mDrawHelper;
        private UWB_BoundingBox mWorldBounds;

        private UWB_Clock mStopWatch;

        public Model()
        {
            mScene = new UWB_SceneNode("Scene");
            UWB_SceneNode body = new UWB_SceneNode("Body");
            mScene.insertChildNode(body);
            mActiveNode = mScene;

            UWB_PrimitiveList bodyPrim = new UWB_PrimitiveList();
            UWB_PrimitiveRectangle rec = new UWB_PrimitiveRectangle();
            rec.setCorners(new Vector3(-1.0f, -5.0f, 0.0f), new Vector3(1.0f, 5.0f, 0.0f));
            rec.setFlatColor(new Color(50, 50, 255));
            rec.setShadeMode(eShadeMode.smFlat);
            bodyPrim.append(rec);

            UWB_PrimitiveCircle circle = new UWB_PrimitiveCircle();
            circle.setCenter(0.0f, 0.0f, 0.0f);
            circle.setRadius(1.75f);
            circle.setFlatColor(new Color(0, 0, 100, 100));
            circle.setShadeMode(eShadeMode.smFlat);
            bodyPrim.append(circle);
            body.setPrimitive(bodyPrim);

            CArm l = new CArm(new Vector3(1.0f, 4.0f, 0.0f), "LeftArm");
            CArm r = new CArm(new Vector3(1.0f, -4.0f, 0.0f), "RightArm");
            mLeftPalmNode = l.GetPalmNode();
            mRightPalmNode = r.GetPalmNode();
            body.insertChildNode(l);
            body.insertChildNode(r);

            UWB_XFormInfo xf = body.getXFormInfo();
            xf.setRotationDegrees(90.0f);
            xf.setTranslation(new Vector3(0.0f, -6.0f, 0.0f));
            body.setXFormInfo(xf);

            // A couple of lines and a couple of points to show texture mapping on lines/points
            UWB_SceneNode ln = new UWB_SceneNode("Line Node");
            mScene.insertChildNode(ln);
            UWB_PrimitiveList list = new UWB_PrimitiveList();
            UWB_PrimitiveLine lines = new UWB_PrimitiveLine();
            lines.setStartPoint(1, 1, 0);
            lines.setEndPoint(5, 5, 0);
            list.append(lines);
            lines = new UWB_PrimitiveLine();
            lines.setStartPoint(2, 5, 0);
            lines.setEndPoint(10, 10, 0);
            list.append(lines);
            ln.setPrimitive(list);

            ln = new UWB_SceneNode("Point Node");
            mScene.insertChildNode(ln);
            list = new UWB_PrimitiveList();
            UWB_PrimitivePoint pt = new UWB_PrimitivePoint();
            pt.setPointSize(3.0f);
            pt.setLocation(0.0f, 0.0f, 0.0f);
            list.append(pt);
            pt = new UWB_PrimitivePoint();
            pt.setPointSize(2.0f);
            pt.setLocation(4.0f, 8.0f, 0.0f);
            list.append(pt);
            ln.setPrimitive(list);
            

            mBullets = new UWB_PrimitiveList();
            mDrawHelper = new UWB_XNADrawHelper();
            mStopWatch = new UWB_Clock();
            mWorldBounds = new UWB_BoundingBox(new Vector3(-10, -10, 0), new Vector3(10, 10, 0));
        }

        public UWB_SceneNode GetRootNode()
        {
            return mScene;
        }

        public void updateSimulation()
        {
            float elapsed_seconds = mStopWatch.getSecondsElapsed();

            //check for collisions between all of the existing balls and the SceneNode object
            //Balls that hit the palms should bounce
            //Balls that hit body count as a hit

            int count = mBullets.count();
            eLevelofDetail lod = mDrawHelper.getLod();

            UWB_BoundingBox left_palm_bounds = new UWB_BoundingBox();
            UWB_BoundingBox right_palm_bounds = new UWB_BoundingBox();

            mDrawHelper.initializeModelTransform();
            mScene.getNodeBounds(mLeftPalmNode, ref left_palm_bounds, mDrawHelper, false);
            mScene.getNodeBounds(mRightPalmNode, ref right_palm_bounds, mDrawHelper, false);

            // walk backward through list so we can remove balls when a hit occurs
            int hits = mHitCount;
            for (int i = count - 1; i >= 0; i--)
            {
                UWB_Primitive testPrimitive = mBullets.primitiveAt(i);
                if (null == testPrimitive)
                    continue;

                testPrimitive.update(elapsed_seconds);

                UWB_BoundingVolume boundVolume = testPrimitive.getBoundingVolume(lod);
                if (null == boundVolume)
                {
                    mBullets.deletePrimitiveAt(i);
                    continue;
                }

                if (left_palm_bounds.intersects(boundVolume))
                {
                    testPrimitive.collisionResponse(mLeftPalmNode.getPrimitive(), left_palm_bounds.getCenter());
                    mHitCount++;
                    continue;
                }
                if (right_palm_bounds.intersects(boundVolume))
                {
                    testPrimitive.collisionResponse(mRightPalmNode.getPrimitive(), right_palm_bounds.getCenter());
                    mHitCount++;
                    continue;
                }
            }
            if (hits != mHitCount)
            {
                String msg = "Hit Count = " + mHitCount;
                UWB_Utility.echoToStatusArea(msg);
            }

            mSecondCounter += elapsed_seconds;
            if (mSecondCounter > 0.5f)
            {
                mSecondCounter = 0.0f;
                CreateNewBullet();
            }

            //destroy balls that are outside the world
            count = mBullets.count();
            for (int i = count - 1; i >= 0; i--)
            {
                UWB_Primitive aBullet = mBullets.primitiveAt(i);
                if (null == aBullet)
                    continue;

                UWB_BoundingVolume bV = aBullet.getBoundingVolume(lod);
                if (null == bV || !bV.intersects(mWorldBounds))
                    mBullets.deletePrimitiveAt(i);
            }

            //------------  now update the rotation of the two palms ...

            UWB_XFormInfo xform = mLeftPalmNode.getXFormInfo();
            float rotation = xform.getRotationInDegrees();
            xform.setPivot(new Vector3(5.0f, 4.0f, 0.0f));
            rotation += 10;
            if (rotation > 360)
                rotation -= 360.0f;

            xform.setRotationDegrees(rotation);
            mLeftPalmNode.setXFormInfo(xform);

            xform = mRightPalmNode.getXFormInfo();
            rotation = xform.getRotationInDegrees();
            xform.setPivot(new Vector3(5.0f, -4.0f, 0.0f));
            rotation -= 15;
            if (rotation > 360)
                rotation -= 360.0f;

            xform.setRotationDegrees(rotation);
            mRightPalmNode.setXFormInfo(xform);
        }

        public void getWorldBounds(ref float width, ref float height)
        {
            width = mWorldBounds.width();
            height = mWorldBounds.height();
        }

        public UWB_BoundingBox getWorldBounds()
        {
            return mWorldBounds;
        }

        public void drawGrid(UWB_DrawHelper drawHelper, UWB_BoundingBox bounds)
        {
            Color gridColor = new Color(150, 150, 150);
            Color axisColor = new Color(130, 30, 30);

            drawHelper.setColor1(gridColor);
            drawHelper.setColor1(gridColor);

            Vector3 minPoint = bounds.getMin();
            Vector3 maxPoint = bounds.getMax();

            float y = minPoint.Y;
            while (y < maxPoint.Y)
            {
                if (y == 0)
                {
                    drawHelper.setColor1(axisColor);
                    drawHelper.setColor2(axisColor);
                    drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
                    drawHelper.setColor1(gridColor);
                    drawHelper.setColor2(gridColor);
                }
                else
                {
                    drawHelper.setColor1(gridColor);
                    drawHelper.setColor2(gridColor);
                    drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
                }
                y += 1;
            }

            float x = minPoint.X;
            while (x < maxPoint.X)
            {
                if (x == 0)
                {
                    drawHelper.setColor1(axisColor);
                    drawHelper.setColor2(axisColor);
                    drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
                    drawHelper.setColor1(gridColor);
                    drawHelper.setColor2(gridColor);
                }
                else
                {
                    drawHelper.setColor1(gridColor);
                    drawHelper.setColor2(gridColor);
                    drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
                }
                x += 1;
            }
        }

        public UWB_BoundingBox getWorldBound()
        { return mWorldBounds; }

        public void drawModel()
        {
            mDrawHelper.initializeModelTransform();

            drawGrid(mDrawHelper, mWorldBounds);

            mDrawHelper.pushModelTransform();

               eLevelofDetail lod = mDrawHelper.getLod();
               mScene.draw(lod, mDrawHelper);

            mDrawHelper.popModelTransform();

            mBullets.draw(lod, mDrawHelper);
        }

        public void moveCharacterX(float x)
        {
            UWB_XFormInfo xform = mScene.getXFormInfo();
            Vector3 t = xform.getTranslation();
            t.X = x;
            xform.setTranslation(t);
            mScene.setXFormInfo(xform);
        }

        private void CreateNewBullet()
        {
            Vector3 minpt = mWorldBounds.getMin();
            Vector3 maxpt = mWorldBounds.getMax();
            float x = UWB_Utility.UWB_RandomNumber(minpt.X, maxpt.X);

            float r = UWB_Utility.UWB_RandomNumber(0.0f, 5.0f);
            UWB_Primitive obj;
            if (r > 2.0f)
                obj = new CBall(x, maxpt.Y, 0);
            else
                obj = new CPrimitiveArm(x, maxpt.Y, 0);

            mBullets.append(obj);
        }
        
        public void setActiveNode(UWB_SceneNode n)
        {
            mActiveNode = n;
        }
        public UWB_SceneNode getActiveNode()
        {
            return mActiveNode;
        }

    }
}

